@@ -1,7 +1,7 @@
 # ducktools-pythonfinder
 # MIT License
 #
-# Copyright (c) 2023-2024 David C Ellis
+# Copyright (c) 2023-2025 David C Ellis
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,18 +22,27 @@
 # SOFTWARE.
 from __future__ import annotations
 
-
 import sys
 import os
 
 from ducktools.lazyimporter import LazyImporter, ModuleImport, FromImport
-from ducktools.pythonfinder import list_python_installs, __version__
+
+from . import list_python_installs, __version__
+from .shared import purge_caches, version_str_to_tuple
+
+
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    import argparse
+
 
 _laz = LazyImporter(
     [
         ModuleImport("argparse"),
         ModuleImport("csv"),
+        ModuleImport("re"),
         ModuleImport("subprocess"),
+        ModuleImport("sysconfig"),
         ModuleImport("platform"),
         FromImport(".pythonorg_search", "PythonOrgSearch"),
         FromImport("packaging.specifiers", "SpecifierSet"),
@@ -47,7 +56,7 @@ class UnsupportedPythonError(Exception):
     pass
 
 
-def stop_autoclose():
+def stop_autoclose() -> None:
     """
     Checks if it thinks windows will auto close the window after running
 
@@ -74,11 +83,11 @@ def stop_autoclose():
         _laz.subprocess.run("pause", shell=True)
 
 
-def _get_parser_class():
+def _get_parser_class() -> type[argparse.ArgumentParser]:
     # This class is deferred to avoid the argparse import
     # if there are no arguments to parse
 
-    class FixedArgumentParser(_laz.argparse.ArgumentParser):
+    class FixedArgumentParser(_laz.argparse.ArgumentParser):  # type: ignore
         """
         The builtin argument parser uses shutil to figure out the terminal width
         to display help info. This one replaces the function that calls help info
@@ -106,7 +115,7 @@ def _get_parser_class():
     return FixedArgumentParser
 
 
-def get_parser():
+def get_parser() -> argparse.ArgumentParser:
     FixedArgumentParser = _get_parser_class()  # noqa
 
     parser = FixedArgumentParser(
@@ -114,14 +123,14 @@ def get_parser():
         description="Discover base Python installs",
     )
     parser.add_argument("-V", "--version", action="version", version=__version__)
-    parser.add_argument(
-        "--fast",
-        action="store_true",
-        help="Skip Python installs that need to be launched to obtain metadata"
-    )
 
     subparsers = parser.add_subparsers(dest="command", required=False)
 
+    clear_cache = subparsers.add_parser(
+        "clear-cache",
+        help="Clear the cache of Python install details"
+    )
+
     online = subparsers.add_parser(
         "online",
         help="Get links to binaries from python.org"
@@ -160,19 +169,18 @@ def get_parser():
 
 
 def display_local_installs(
-    min_ver=None,
-    max_ver=None,
-    compatible=None,
-    query_executables=True,
-):
+    min_ver: str | None = None,
+    max_ver: str | None = None,
+    compatible: str | None = None,
+) -> None:
     if min_ver:
-        min_ver = tuple(int(i) for i in min_ver.split("."))
+        min_ver_tuple = version_str_to_tuple(min_ver)
     if max_ver:
-        max_ver = tuple(int(i) for i in max_ver.split("."))
+        max_ver_tuple = version_str_to_tuple(max_ver)
     if compatible:
-        compatible = tuple(int(i) for i in compatible.split("."))
+        compatible_spec = _laz.SpecifierSet(f"~={compatible}")
 
-    installs = list_python_installs(query_executables=query_executables)
+    installs = list_python_installs()
 
     headings = ["Version", "Executable Location"]
 
@@ -184,16 +192,12 @@ def display_local_installs(
 
     # First collect the strings
     for install in installs:
-        if min_ver and install.version < min_ver:
+        if min_ver and install.version < min_ver_tuple:
             continue
-        elif max_ver and install.version > max_ver:
+        elif max_ver and install.version > max_ver_tuple:
+            continue
+        elif compatible and not compatible_spec.contains(install.version_str):
             continue
-        elif compatible:
-            if install.version < compatible:
-                continue
-            version_parts = len(compatible) - 1
-            if install.version[:version_parts] > compatible[:-1]:
-                continue
 
         version_str = install.version_str
 
@@ -207,7 +211,7 @@ def display_local_installs(
             version_str = f"*{version_str}"
         elif (
             sys.prefix != sys.base_prefix
-            and os.path.commonpath([install.executable, sys.base_prefix]) == sys.base_prefix
+            and install.paths.get("stdlib") == _laz.sysconfig.get_path("stdlib")
         ):
             version_str = f"**{version_str}"
 
@@ -240,7 +244,7 @@ def display_local_installs(
     if sys.platform != "win32":
         print("[] - This Python install is shadowed by another on Path")
     print("* - This is the active Python executable used to call this module")
-    print("** - This is the parent Python executable of the venv used to call this module")
+    print("** - This is a parent Python executable of the venv used to call this module")
     print()
 
     headings_str = f"| {headings[0]:<{max_version_len}s} | {headings[1]:<{max_executable_len}s} |"
@@ -252,14 +256,14 @@ def display_local_installs(
 
 
 def display_remote_binaries(
-    min_ver,
-    max_ver,
-    compatible,
-    all_binaries,
-    system,
-    machine,
-    prerelease
-):
+    min_ver: str,
+    max_ver: str,
+    compatible: str,
+    all_binaries: bool,
+    system: str,
+    machine: str,
+    prerelease: bool,
+) -> None:
     specs = []
     if min_ver:
         specs.append(f">={min_ver}")
@@ -293,19 +297,21 @@ def display_remote_binaries(
         print("No Python releases found matching specification")
 
 
-def main():
-    if sys.version_info < (3, 8):
+def main() -> int:
+    if sys.version_info < (3, 10):
         v = sys.version_info
         raise UnsupportedPythonError(
             f"Python {v.major}.{v.minor}.{v.micro} is not supported. "
-            f"ducktools.pythonfinder requires Python 3.8 or later."
+            f"ducktools.pythonfinder requires Python 3.10 or later."
         )
 
     if sys.argv[1:]:
         parser = get_parser()
         vals = parser.parse_args(sys.argv[1:])
 
-        if vals.command == "online":
+        if vals.command == "clear-cache":
+            purge_caches()
+        elif vals.command == "online":
             system = vals.system if vals.system else _laz.platform.system()
             machine = vals.machine if vals.machine else _laz.platform.machine()
             try:
@@ -325,7 +331,6 @@ def main():
                 min_ver=vals.min,
                 max_ver=vals.max,
                 compatible=vals.compatible,
-                query_executables=not vals.fast,
             )
     else:
         # No arguments to parse
@@ -333,6 +338,8 @@ def main():
 
     stop_autoclose()
 
+    return 0
+
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
