@@ -1,7 +1,7 @@
 # ducktools-pythonfinder
 # MIT License
 #
-# Copyright (c) 2023-2024 David C Ellis
+# Copyright (c) 2023-2025 David C Ellis
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,11 @@ def version_str_to_tuple(version):
     import re
 
     parsed_version = re.fullmatch(FULL_PY_VER_RE, version)
+    if parsed_version is None:
+        raise ValueError(
+            "'version' must be a valid Python version string, "
+            + "not {version!r}".format(version=version)
+        )
 
     major, minor, micro, releaselevel, serial = parsed_version.groups()
 
@@ -41,7 +46,7 @@ def version_str_to_tuple(version):
         releaselevel = "alpha"
     elif releaselevel == "b":
         releaselevel = "beta"
-    elif releaselevel == "rc":
+    elif releaselevel in {"c", "rc"}:
         releaselevel = "candidate"
     else:
         releaselevel = "final"
@@ -70,14 +75,19 @@ def get_details():
             # Special case GraalPy as it erroneously reports the CPython target
             # instead of the Graal versiion
             try:
-                ver = __graalpython__.get_graalvm_version()
+                ver = __graalpython__.get_graalvm_version()  # type: ignore
                 metadata = {
                     "graalpy_version": version_str_to_tuple(ver)
                 }
-            except NameError:
-                metadata = {"{}_version".format(implementation): sys.implementation.version}
+            except (NameError, ValueError):
+                metadata = {"{}_version".format(implementation): sys.implementation.version}            
         elif implementation != "cpython":  # pragma: no cover
-            metadata = {"{}_version".format(implementation): sys.implementation.version}
+            if implementation == "micropython":
+                imp_ver = sys.implementation.version[:3]
+            else:
+                imp_ver = sys.implementation.version
+
+            metadata = {"{}_version".format(implementation): imp_ver}
         else:
             metadata = {}
             if sys.version_info >= (3, 13):
@@ -85,13 +95,29 @@ def get_details():
                 freethreaded = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))
                 metadata["freethreaded"] = freethreaded
 
+    # Attempt to get the paths to stdlib etc from sysconfig
+    try:
+        import sysconfig  # Exists in 2.7 and 3.2 onwards, missing in earlier and micropython
+    except ImportError:
+        paths = {}
+    else:
+        paths = sysconfig.get_paths()
 
+    # Use struct to correctly identify GraalPy as 64 bit
+    try:
+        import struct
+    except ImportError:
+        architecture = "64bit" if (sys.maxsize > 2**32) else "32bit"
+    else:
+        architecture = "32bit" if (struct.calcsize("P") == 4) else "64bit"
+    
     install = dict(
         version=list(sys.version_info),
         executable=sys.executable,
-        architecture="64bit" if (sys.maxsize > 2**32) else "32bit",
+        architecture=architecture,
         implementation=implementation,
         metadata=metadata,
+        paths=paths,
     )
 
     return install
