@@ -1,7 +1,7 @@
 # ducktools-pythonfinder
 # MIT License
 #
-# Copyright (c) 2023-2024 David C Ellis
+# Copyright (c) 2023-2025 David C Ellis
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -25,21 +25,47 @@ from __future__ import annotations
 import os
 import os.path
 import itertools
-from _collections_abc import Iterator
 
-from ..shared import PythonInstall, get_folder_pythons, get_uv_pythons, get_uv_python_path
-from .pyenv_search import get_pyenv_pythons
+try:
+    from _collections_abc import Iterator
+except ImportError:
+    from collections.abc import Iterator
+
+from ..shared import (
+    DetailFinder,
+    PythonInstall,
+    get_folder_pythons,
+    get_uv_pythons,
+    get_uv_python_path
+)
+from .pyenv_search import get_pyenv_pythons, get_pyenv_root
 
 
-def get_path_pythons() -> Iterator[PythonInstall]:
+KNOWN_MANAGED_PATHS = {
+    "/usr/bin": "OS",
+    "/bin": "OS",
+    "/usr/sbin": "OS",
+    "/sbin": "OS",
+}
+
+
+def get_path_pythons(
+    *,
+    finder: DetailFinder | None = None,
+    known_paths: dict[str, str] | None = None,
+) -> Iterator[PythonInstall]:
+
     exe_names = set()
 
     path_folders = os.environ.get("PATH", "").split(":")
-    pyenv_root = os.environ.get("PYENV_ROOT")
+    pyenv_root = get_pyenv_root()
     uv_root = get_uv_python_path()
 
     excluded_folders = [pyenv_root, uv_root]
 
+    finder = DetailFinder() if finder is None else finder
+    known_paths = KNOWN_MANAGED_PATHS if known_paths is None else known_paths
+
     for fld in path_folders:
         # Don't retrieve pyenv installs
         skip_folder = False
@@ -54,7 +80,12 @@ def get_path_pythons() -> Iterator[PythonInstall]:
         if not os.path.exists(fld):
             continue
 
-        for install in get_folder_pythons(fld):
+        for install in get_folder_pythons(fld, finder=finder):
+            for path, manager in known_paths.items():
+                if os.path.commonpath((path, install.executable)) == path:
+                    install.managed_by = manager
+                    break
+
             name = os.path.basename(install.executable)
             if name in exe_names:
                 install.shadowed = True
@@ -63,17 +94,21 @@ def get_path_pythons() -> Iterator[PythonInstall]:
             yield install
 
 
-def get_python_installs(*, query_executables=True):
+def get_python_installs(
+    *,
+    finder: DetailFinder | None = None,
+) -> Iterator[PythonInstall]:
     listed_pythons = set()
 
-    chain_commands = [
-        get_pyenv_pythons(query_executables=query_executables),
-        get_uv_pythons(query_executables=query_executables),
-    ]
-    if query_executables:
-        chain_commands.append(get_path_pythons())
+    finder = DetailFinder() if finder is None else finder
 
-    for py in itertools.chain.from_iterable(chain_commands):
-        if py.executable not in listed_pythons:
-            yield py
-            listed_pythons.add(py.executable)
+    chain_commands = [
+        get_pyenv_pythons(finder=finder),
+        get_uv_pythons(finder=finder),
+        get_path_pythons(finder=finder),
+    ]
+    with finder:
+        for py in itertools.chain.from_iterable(chain_commands):
+            if py.executable not in listed_pythons:
+                yield py
+                listed_pythons.add(py.executable)
