@@ -1,7 +1,7 @@
 # ducktools-pythonfinder
 # MIT License
 #
-# Copyright (c) 2023-2024 David C Ellis
+# Copyright (c) 2023-2025 David C Ellis
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,9 +26,12 @@ import sys
 import os
 import os.path
 
-from _collections_abc import Iterator
+try:
+    from _collections_abc import Iterator
+except ImportError:
+    from collections.abc import Iterator
 
-from ducktools.classbuilder.prefab import Prefab, attribute
+from ducktools.classbuilder.prefab import Prefab, attribute, as_dict
 from ducktools.lazyimporter import LazyImporter, ModuleImport, FromImport
 
 from . import details_script
@@ -39,13 +42,13 @@ _laz = LazyImporter(
         ModuleImport("json"),
         ModuleImport("platform"),
         ModuleImport("re"),
+        ModuleImport("shutil"),
         ModuleImport("subprocess"),
         ModuleImport("tempfile"),
         ModuleImport("zipfile"),
     ]
 )
 
-
 FULL_PY_VER_RE = r"(?P<major>\d+)\.(?P<minor>\d+)\.?(?P<micro>\d*)-?(?P<releaselevel>a|b|c|rc)?(?P<serial>\d*)?"
 
 UV_PYTHON_RE = (
@@ -56,6 +59,38 @@ UV_PYTHON_RE = (
     r"-.*"
 )
 
+# Cache for runtime details
+# Code to work out where to store data
+# Store in LOCALAPPDATA for windows, User folder for other operating systems
+if sys.platform == "win32":
+    # os.path.expandvars will actually import a whole bunch of other modules
+    # Try just using the environment.
+    if _local_app_folder := os.environ.get("LOCALAPPDATA"):
+        if not os.path.isdir(_local_app_folder):
+            raise FileNotFoundError(
+                f"Could not find local app data folder {_local_app_folder}"
+            )
+    else:
+        raise EnvironmentError(
+            "Environment variable %LOCALAPPDATA% "
+            "for local application data folder location "
+            "not found"
+        )
+    USER_FOLDER = _local_app_folder
+    CACHE_FOLDER = os.path.join(USER_FOLDER, "ducktools", "pythonfinder", "cache")
+else:
+    USER_FOLDER = os.path.expanduser("~")
+    CACHE_FOLDER = os.path.join(USER_FOLDER, ".cache", "ducktools", "pythonfinder")
+
+
+CACHE_VERSION = 2
+DETAILS_CACHE_PATH = os.path.join(CACHE_FOLDER, f"runtime_cache_v{CACHE_VERSION}.json")
+INSTALLER_CACHE_PATH = os.path.join(CACHE_FOLDER, "installer_details.json")
+
+
+def purge_caches(cache_folder=CACHE_FOLDER):
+    _laz.shutil.rmtree(cache_folder, ignore_errors=True)
+
 
 def version_str_to_tuple(version):
     parsed_version = _laz.re.fullmatch(FULL_PY_VER_RE, version)
@@ -129,7 +164,157 @@ class DetailsScript(Prefab):
         return self._source_code
 
 
-details = DetailsScript()
+class DetailFinder(Prefab):
+    cache_path: str = DETAILS_CACHE_PATH
+    details_script: DetailsScript = attribute(default_factory=DetailsScript)
+
+    # Stores the dict loaded from the JSON file without processing
+    _raw_cache: dict | None = attribute(default=None, private=True)
+
+    # Indicates if the cache is known to have changed
+    _dirty_cache: bool = attribute(default=False, private=True)
+
+    # Increased each re-entry to the context manager
+    # Decreased on exit
+    # Save should only occur when all contexts exit
+    _context_level: int = attribute(default=0, private=True)
+
+    def __enter__(self):
+        self._context_level += 1
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._context_level -= 1
+        if (
+            exc_type in {None, GeneratorExit}
+            and self._dirty_cache
+            and self._context_level == 0
+        ):
+            self.save()
+
+    @property
+    def raw_cache(self) -> dict:
+        if self._raw_cache is None:
+            try:
+                with open(self.cache_path) as f:
+                    self._raw_cache = _laz.json.load(f)
+            except (_laz.json.JSONDecodeError, FileNotFoundError):
+                self._raw_cache = {}
+        return self._raw_cache
+
+    def save(self) -> None:
+        os.makedirs(os.path.dirname(self.cache_path), exist_ok=True)
+        with open(self.cache_path, 'w') as f:
+            _laz.json.dump(self.raw_cache, f, indent=4)
+
+        self._dirty_cache = False
+
+    def clear_invalid_runtimes(self) -> None:
+        """
+        Remove cache entries where the python.exe no longer exists
+        """
+        removed_runtimes: set[str] = set()
+        for exe_path in self.raw_cache.copy().keys():
+            if not os.path.exists(exe_path):
+                self.raw_cache.pop(exe_path)
+                removed_runtimes.add(exe_path)
+        if removed_runtimes:
+            self._dirty_cache = True
+
+    def clear_cache(self) -> None:
+        """
+        Completely empty the cache
+        """
+        self._raw_cache = {}
+        self._dirty_cache = True
+
+    def query_install(
+        self,
+        exe_path: str,
+        managed_by: str | None = None,
+        metadata: dict | None = None,
+    ) -> PythonInstall | None:
+        """
+        Query the details of a Python install directly
+
+        :param exe_path: Path to the runtime .exe
+        :param managed_by: Which tool manages this install (if any)
+        :param metadata: Dictionary of install metadata
+        :return: a PythonInstall if one exists at the exe Path
+        """
+        try:
+            source = self.details_script.get_source_code()
+        except FileNotFoundError:
+            return None
+
+        try:
+            detail_output = _laz.subprocess.run(
+                [exe_path, "-"],
+                input=source,
+                capture_output=True,
+                text=True,
+                check=True,
+            ).stdout
+        except OSError:
+            # Something else has gone wrong
+            return None
+        except (_laz.subprocess.CalledProcessError, FileNotFoundError):
+            # Potentially this is micropython which does not support
+            # piping from stdin. Try using a file in a temporary folder.
+            # Python 3.12 has delete_on_close that would make TemporaryFile
+            # Usable on windows but for now use a directory
+            with _laz.tempfile.TemporaryDirectory() as tempdir:
+                temp_script = os.path.join(tempdir, "details_script.py")
+                with open(temp_script, "w") as f:
+                    f.write(source)
+                try:
+                    detail_output = _laz.subprocess.run(
+                        [exe_path, temp_script],
+                        capture_output=True,
+                        text=True,
+                        check=True,
+                    ).stdout
+                except (_laz.subprocess.CalledProcessError, FileNotFoundError):
+                    return None
+
+        try:
+            output = _laz.json.loads(detail_output)
+        except _laz.json.JSONDecodeError:
+            return None
+
+        if metadata:
+            output["metadata"].update(metadata)
+
+        install = PythonInstall.from_json(**output, managed_by=managed_by)
+
+        return install
+
+    def get_install_details(
+        self,
+        exe_path: str,
+        managed_by: str | None = None,
+        metadata: dict | None = None,
+    ) -> PythonInstall | None:
+        exe_path = os.path.abspath(exe_path)
+        mtime = os.stat(exe_path).st_mtime
+
+        install = None
+        if cached_details := self.raw_cache.get(exe_path):
+            if cached_details["mtime"] == mtime:
+                install = PythonInstall.from_json(**cached_details["install"])
+            else:
+                self.raw_cache.pop(exe_path)
+
+        if install is None:
+            install = self.query_install(exe_path, managed_by, metadata)
+            if install:
+                self.raw_cache[exe_path] = {
+                    "mtime": mtime,
+                    "install": as_dict(install)
+                }
+                self._dirty_cache = True
+
+        return install
 
 
 class PythonInstall(Prefab):
@@ -139,7 +324,8 @@ class PythonInstall(Prefab):
     implementation: str = "cpython"
     managed_by: str | None = None
     metadata: dict = attribute(default_factory=dict)
-    shadowed: bool = False
+    paths: dict[str, str] = attribute(default_factory=dict)
+    shadowed: bool = attribute(default=False, serialize=False)
     _implementation_version: tuple[int, int, int, str, int] | None = attribute(default=None, private=True)
 
     def __prefab_post_init__(
@@ -174,6 +360,8 @@ class PythonInstall(Prefab):
     def implementation_version_str(self) -> str:
         return version_tuple_to_str(self.implementation_version)
 
+    # Typing these classmethods would require an import
+    # This is not acceptable for performance reasons
     @classmethod
     def from_str(
         cls,
@@ -183,10 +371,13 @@ class PythonInstall(Prefab):
         implementation: str = "cpython",
         managed_by: str | None = None,
         metadata: dict | None = None,
+        paths: dict | None = None,
     ):
         version_tuple = version_str_to_tuple(version)
 
-        # noinspection PyArgumentList
+        metadata = {} if metadata is None else metadata
+        paths = {} if paths is None else paths
+
         return cls(
             version=version_tuple,
             executable=executable,
@@ -194,21 +385,33 @@ class PythonInstall(Prefab):
             implementation=implementation,
             managed_by=managed_by,
             metadata=metadata,
+            paths=paths,
         )
 
     @classmethod
-    def from_json(cls, version, executable, architecture, implementation, metadata, managed_by=None):
+    def from_json(
+        cls,
+        version: list[int | str],  # This is actually the list version of [int, int, int, str, int]
+        executable: str,
+        architecture: str,
+        implementation: str,
+        metadata: dict,
+        paths: dict | None = None,
+        managed_by: str | None = None,
+    ):
         if arch_ver := metadata.get(f"{implementation}_version"):
             metadata[f"{implementation}_version"] = tuple(arch_ver)
 
-        # noinspection PyArgumentList
+        paths = {} if paths is None else paths
+
         return cls(
-            version=tuple(version),
+            version=tuple(version),  # type: ignore
             executable=executable,
             architecture=architecture,
             implementation=implementation,
             managed_by=managed_by,
             metadata=metadata,
+            paths=paths,
         )
 
     def get_pip_version(self) -> str | None:
@@ -231,6 +434,7 @@ class PythonInstall(Prefab):
         return pip_call.stdout
 
 
+# Return type missing due to import requirements
 def _python_exe_regex(basename: str = "python"):
     if sys.platform == "win32":
         return _laz.re.compile(rf"{basename}\d?\.?\d*\.exe")
@@ -238,63 +442,25 @@ def _python_exe_regex(basename: str = "python"):
         return _laz.re.compile(rf"{basename}\d?\.?\d*")
 
 
-def get_install_details(executable: str, managed_by=None) -> PythonInstall | None:
-    try:
-        source = details.get_source_code()
-    except FileNotFoundError:
-        return None
-
-    try:
-        detail_output = _laz.subprocess.run(
-            [executable, "-"],
-            input=source,
-            capture_output=True,
-            text=True,
-            check=True,
-        ).stdout
-    except OSError:
-        # Something else has gone wrong
-        return None
-    except (_laz.subprocess.CalledProcessError, FileNotFoundError):
-        # Potentially this is micropython which does not support
-        # piping from stdin. Try using a file in a temporary folder.
-        # Python 3.12 has delete_on_close that would make TemporaryFile
-        # Usable on windows but for now use a directory
-        with _laz.tempfile.TemporaryDirectory() as tempdir:
-            temp_script = os.path.join(tempdir, "details_script.py")
-            with open(temp_script, "w") as f:
-                f.write(source)
-            try:
-                detail_output = _laz.subprocess.run(
-                    [executable, temp_script],
-                    capture_output=True,
-                    text=True,
-                    check=True,
-                ).stdout
-            except (_laz.subprocess.CalledProcessError, FileNotFoundError):
-                return None
-
-    try:
-        output = _laz.json.loads(detail_output)
-    except _laz.json.JSONDecodeError as e:
-        return None
-
-    return PythonInstall.from_json(**output, managed_by=managed_by)
-
-
 def get_folder_pythons(
     base_folder: str | os.PathLike,
-    basenames: tuple[str] = ("python", "pypy")
-):
+    basenames: tuple[str, ...] = ("python", "pypy", "micropython"),
+    finder: DetailFinder | None = None,
+    managed_by: str | None = None,
+) -> Iterator[PythonInstall]:
     regexes = [_python_exe_regex(name) for name in basenames]
 
-    with os.scandir(base_folder) as fld:
+    finder = DetailFinder() if finder is None else finder
+
+    base_folder = str(base_folder)
+
+    with finder, os.scandir(base_folder) as fld:
         for file_path in fld:
             try:
                 is_file = file_path.is_file()
             except PermissionError:
                 continue
-            
+
             if (
                 is_file
                 and any(reg.fullmatch(file_path.name) for reg in regexes)
@@ -302,18 +468,32 @@ def get_folder_pythons(
                 p = file_path.path
                 if file_path.is_symlink():
                     # Might be a venv - look for pyvenv.cfg in parent
-                    fld = os.path.dirname(p)
-                    if os.path.exists(os.path.join(fld, "../pyvenv.cfg")):
+                    dirname = os.path.dirname(p)
+
+                    if os.path.exists(os.path.join(dirname, "../pyvenv.cfg")):
                         continue
                 else:
                     p = file_path.path
-                install = get_install_details(p)
+
+                install = finder.get_install_details(p, managed_by=managed_by)
                 if install:
                     yield install
 
 
 # UV Specific finder
 def get_uv_python_path() -> str | None:
+    # Attempt to get cache
+    try:
+        with open(INSTALLER_CACHE_PATH) as f:
+            installer_cache = _laz.json.load(f)
+    except (FileNotFoundError, _laz.json.JSONDecodeError):
+        installer_cache = {}
+
+    uv_python_dir = installer_cache.get("uv")
+    if uv_python_dir and os.path.exists(uv_python_dir):
+        return uv_python_dir
+
+    # Cache failed
     try:
         uv_python_find = _laz.subprocess.run(
             ["uv", "python", "dir"],
@@ -327,57 +507,51 @@ def get_uv_python_path() -> str | None:
         # remove newline
         uv_python_dir = uv_python_find.stdout.strip()
 
+    # Fill cache and update the cache file
+    installer_cache["uv"] = uv_python_dir
+    os.makedirs(os.path.dirname(INSTALLER_CACHE_PATH), exist_ok=True)
+    with open(INSTALLER_CACHE_PATH, 'w') as f:
+        _laz.json.dump(installer_cache, f)
+
     return uv_python_dir
 
 
 def _implementation_from_uv_dir(
     direntry: os.DirEntry,
-    query_executables: bool = True,
+    finder: DetailFinder | None = None,
 ) -> PythonInstall | None:
-    python_exe = "python.exe" if sys.platform == "win32" else "bin/python"
-    python_path = os.path.join(direntry, python_exe)
+    if sys.platform == "win32":
+        python_paths = [
+            os.path.join(direntry, "python.exe"),
+            os.path.join(direntry, "bin/python.exe"),  # graalpy
+        ]
+    else:
+        python_paths = [
+            os.path.join(direntry, "bin/python")
+        ]
 
     install: PythonInstall | None = None
+    finder = DetailFinder() if finder is None else finder
 
-    if os.path.exists(python_path):
-        if match := _laz.re.fullmatch(UV_PYTHON_RE, direntry.name):
-            implementation, version, extra, platform, arch = match.groups()
-            metadata = {
-                "freethreaded": "freethreaded" in extra,
-            }
-
-            try:
-                if implementation in {"cpython"}:
-                    install = PythonInstall.from_str(
-                        version=version,
-                        executable=python_path,
-                        architecture="32bit" if arch in {"i686", "armv7"} else "64bit",
-                        implementation=implementation,
-                        metadata=metadata,
-                        managed_by="Astral UV",
-                    )
-            except ValueError:
-                pass
-
-        if install is None:
-            # Directory name format has changed or this is an alternate implementation
-            # Slow backup - ask python itself
-            if query_executables:
-                install = get_install_details(python_path)
-                install.managed_by = "Astral UV"
+    for pth in python_paths:
+        if os.path.exists(pth):
+            install = finder.get_install_details(pth, managed_by="Astral")
+            break
 
     return install
 
 
-def get_uv_pythons(query_executables=True) -> Iterator[PythonInstall]:
+def get_uv_pythons(finder=None) -> Iterator[PythonInstall]:
     # This takes some shortcuts over the regular pythonfinder
     # As the UV folders give the python version and the implementation
     if uv_python_path := get_uv_python_path():
         if os.path.exists(uv_python_path):
-            with os.scandir(uv_python_path) as fld:
+            finder = DetailFinder() if finder is None else finder
+
+            with finder, os.scandir(uv_python_path) as fld:
                 for f in fld:
                     if (
                         f.is_dir()
-                        and (install := _implementation_from_uv_dir(f, query_executables))
+                        and (install := _implementation_from_uv_dir(f, finder=finder))
                     ):
                         yield install
